import pandas as pd
from ovretl.containers_utils.calculate_single_container_teus import (
    calculate_single_container_teus,
)


def calculate_entity_containers(containers_df: pd.DataFrame, key: str):
    entity_containers = containers_df[~containers_df[key].isnull()]
    return (
        entity_containers.groupby(key)["teus"]
        .agg(["sum", "count"])
        .rename(columns={"sum": "teus", "count": "tc"})
        .reset_index()
        .drop_duplicates(subset=[key])
    )


def calculate_shipments_propositions_containers(containers_df: pd.DataFrame):
    containers_df = containers_df.apply(calculate_single_container_teus, axis=1)
    containers_df = containers_df.dropna(subset=["teus"])
    propositions_containers = calculate_entity_containers(
        containers_df=containers_df, key="proposition_id"
    )
    shipments_containers = calculate_entity_containers(
        containers_df=containers_df, key="shipment_id"
    )
    return pd.concat([propositions_containers, shipments_containers], sort=False)



import pandas as pd


def merge_with_propositions_loads_then_with_shipments_loads(
    shipments_df: pd.DataFrame,
    propositions_loads_df: pd.DataFrame,
    shipments_loads_df: pd.DataFrame,
) -> pd.DataFrame:
    shipments_df = pd.merge(
        shipments_df,
        propositions_loads_df[["total_volume", "total_weight", "proposition_id"]],
        how="left",
        on="proposition_id",
    )

    shipments_df_with_total_weight = shipments_df[
        ~shipments_df["total_weight"].isnull()
    ]
    shipments_df_without_total_weight = shipments_df[
        shipments_df["total_weight"].isnull()
    ].drop(["total_weight", "total_volume"], axis=1)

    shipments_df_without_total_weight = pd.merge(
        shipments_df_without_total_weight,
        shipments_loads_df[["total_volume", "total_weight", "shipment_id"]],
        how="left",
        on="shipment_id",
    )

    return pd.concat(
        [shipments_df_with_total_weight, shipments_df_without_total_weight], sort=False
    ).reset_index(drop=True)


def split_propositions_shipments_loads(loads_df: pd.DataFrame):
    propositions_loads_df = loads_df[~loads_df["proposition_id"].isnull()]
    shipments_loads_df = loads_df[~loads_df["shipment_id"].isnull()]
    return propositions_loads_df, shipments_loads_df


def merge_shipments_with_loads(shipments_df: pd.DataFrame, loads_df: pd.DataFrame):
    (propositions_loads_df, shipments_loads_df,) = split_propositions_shipments_loads(
        loads_df
    )
    shipments_df = merge_with_propositions_loads_then_with_shipments_loads(
        shipments_df=shipments_df,
        propositions_loads_df=propositions_loads_df,
        shipments_loads_df=shipments_loads_df,
    )
    shipments_df = shipments_df.sort_values(
        by=["total_weight", "total_volume"], ascending=False
    ).drop_duplicates(subset=["shipment_id"])
    return shipments_df



import pandas as pd
from ovretl.billings_utils.extract_billing_numbers import extract_billing_numbers


def test_extract_billing_numbers():
    billings_df = pd.DataFrame(
        data={"shipment_id": ["0", "1", "0"], "billing_number": ["ABC", "DEF", "GHI"]}
    )
    billing_numbers = extract_billing_numbers(billings_df, "0")
    assert billing_numbers == "ABC, GHI"



import pandas as pd


def find_shipment_category_shipowner(role: str, shipowners_associated_df: pd.DataFrame):
    mask_role = shipowners_associated_df["role"] == role
    shipowners_associated_df_filtered = shipowners_associated_df[mask_role].reset_index(
        drop=True
    )
    if len(shipowners_associated_df_filtered) > 0:
        return shipowners_associated_df_filtered.loc[0, "name"]
    return None



import pandas as pd


def find_shipment_employee_name(employees_associated_df: pd.DataFrame, role: str):
    if len(employees_associated_df) == 0:
        return None
    mask_employees = employees_associated_df["role"] == role
    employees_associated_df_filtered = employees_associated_df[
        mask_employees
    ].reset_index(drop=True)
    if len(employees_associated_df_filtered) > 0:
        return employees_associated_df_filtered.loc[0, "name"]
    return None


def add_employees_to_shipment(
    shipment: pd.Series, employees_shipment_with_name_df: pd.DataFrame
):
    mask_shipment = (
        employees_shipment_with_name_df["shipment_id"] == shipment["shipment_id"]
    )
    shipment["sales"] = find_shipment_employee_name(
        employees_associated_df=employees_shipment_with_name_df[mask_shipment],
        role="sales_owner",
    )
    shipment["operations"] = find_shipment_employee_name(
        employees_associated_df=employees_shipment_with_name_df[mask_shipment],
        role="operations_owner",
    )
    return shipment



import pandas as pd


def compute_billing_status(billings_df: pd.DataFrame, shipment_id: str):
    df_billings_shipment = billings_df[billings_df["shipment_id"] == shipment_id]
    if len(df_billings_shipment) == 0:
        return "awaiting_invoice"
    if (
        df_billings_shipment["status"]
        .apply(lambda s: s in ["new", "in_modification"])
        .all()
    ):
        return "awaiting_invoice"
    if df_billings_shipment["status"].apply(lambda s: s == "due").any():
        return "due"
    if df_billings_shipment["status"].apply(lambda s: s == "available").any():
        return "available"
    return "paid"



import pandas as pd


def extract_event_date_by_sorting(
    events_shipment_df: pd.DataFrame, ascending: bool, location_key=None
):
    if len(events_shipment_df) == 0:
        return None
    events_shipment = events_shipment_df.sort_values(
        by="date", ascending=ascending
    ).reset_index(drop=True)
    if len(events_shipment) > 0 and location_key is not None:
        if events_shipment.loc[0, "location_type"] == location_key:
            return events_shipment.loc[0, "date"]
        return None
    if len(events_shipment) > 0:
        return events_shipment.loc[0, "date"]
    return None


def extract_pickup_event_date(events_shipment_df: pd.DataFrame):
    mask_event = events_shipment_df["event_description"].apply(lambda e: "Pickup" in e)
    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event], ascending=True,
    )

    event_found_by_order = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df, ascending=True, location_key="warehouse",
    )

    return event_found_by_description or event_found_by_order


def extract_departure_event_date(events_shipment_df: pd.DataFrame):
    mask_etd = events_shipment_df["is_used_for_etd"] == True
    mask_event = events_shipment_df["event_description"].apply(
        lambda e: "Departure" in e or "Vessel Departure" in e
    )
    event_found_by_etd = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_etd], ascending=True,
    )

    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event], ascending=True,
    )

    event_first = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df, ascending=True, location_key="harbor",
    )
    return event_found_by_etd or event_found_by_description or event_first


def extract_arrival_event_date(events_shipment_df: pd.DataFrame):
    mask_eta = events_shipment_df["is_used_for_eta"] == True
    mask_event = events_shipment_df["event_description"].apply(
        lambda e: "Arrival" in e or "Vessel Arrival" in e
    )
    events_shipment = (
        events_shipment_df[mask_eta]
        .sort_values(by="date", ascending=False)
        .reset_index(drop=True)
    )
    if len(events_shipment) > 0:
        return events_shipment.loc[0, "date"]

    event_found_by_eta = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_eta], ascending=False,
    )

    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event], ascending=False,
    )

    event_last = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df, ascending=False, location_key="harbor",
    )
    return event_found_by_eta or event_found_by_description or event_last


def extract_delivery_event_date(events_shipment_df: pd.DataFrame):
    mask_event = events_shipment_df["event_description"].apply(
        lambda e: "Delivery" in e
    )

    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event], ascending=False,
    )

    event_last = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df,
        ascending=False,
        location_key="warehouse",
    )

    return event_found_by_description or event_last


def add_tracking_event_dates(
    shipment: pd.Series, events_shipment_df: pd.DataFrame
) -> pd.Series:
    mask_shipment = events_shipment_df["shipment_id"] == shipment["shipment_id"]
    events_df_filtered = events_shipment_df[mask_shipment]
    shipment["pickup_date"] = extract_pickup_event_date(events_df_filtered)
    shipment["departure_date"] = extract_departure_event_date(events_df_filtered)
    shipment["arrival_date"] = extract_arrival_event_date(events_df_filtered)
    shipment["delivery_date"] = extract_delivery_event_date(events_df_filtered)
    return shipment



import pandas as pd

from ovretl.tracking_utils.extract_event_date import (
    extract_pickup_event_date,
    extract_arrival_event_date,
    add_tracking_event_dates,
)


def test_extract_pickup_event():
    pickup_events_shipment_df = pd.DataFrame.from_records(
        [
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "actual",
                "date": pd.Timestamp("2020-01-02 23:00:00"),
                "location_type": "warehouse",
            },
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "estimated",
                "date": pd.Timestamp("2015-01-05 23:00:00"),
                "location_type": "warehouse",
            },
        ]
    )
    pickup_date = extract_pickup_event_date(
        events_shipment_df=pickup_events_shipment_df,
    )
    assert pickup_date == pd.Timestamp("2015-01-05 23:00:00")


def test_extract_arrival_event():
    arrival_events_shipment_df = pd.DataFrame.from_records(
        [
            {
                "shipment_id": 0,
                "event_description": "Arrival",
                "date_type": "actual",
                "date": pd.Timestamp("2020-01-02 23:00:00"),
                "is_used_for_eta": False,
                "location_type": "harbor",
            },
            {
                "shipment_id": 0,
                "event_description": "Arrival",
                "date_type": "estimated",
                "date": pd.Timestamp("2015-01-05 23:00:00"),
                "is_used_for_eta": True,
                "location_type": "harbor",
            },
        ]
    )
    arrival_date = extract_arrival_event_date(
        events_shipment_df=arrival_events_shipment_df,
    )
    assert arrival_date == pd.Timestamp("2015-01-05 23:00:00")


def test_add_tracking_events():
    events_shipment_df = pd.DataFrame.from_records(
        [
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "actual",
                "date": pd.Timestamp("2020-01-02 23:00:00"),
                "location_type": "warehouse",
                "is_used_for_etd": False,
                "is_used_for_eta": False,
            },
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "estimated",
                "date": pd.Timestamp("2015-01-05 23:00:00"),
                "location_type": "warehouse",
                "is_used_for_etd": False,
                "is_used_for_eta": False,
            },
        ]
    )
    shipment = pd.Series(data=[0], index=["shipment_id"])
    assert add_tracking_event_dates(shipment, events_shipment_df).equals(shipment)



import pandas as pd
from ovretl.prices_utils.sum_category_price import sum_category_price


def test_sum_category_prices():
    df = pd.DataFrame(
        data={
            "category": [
                "departure_truck_freight",
                "arrival_truck_freight",
                "departure_fees",
                "freight",
                "arrival_fees",
            ],
            "price_in_eur": [1336.05, 680, 108.59, 3651, 95],
            "margin_price_in_eur": [0, 20, 32.49, 534.42, 0],
            "vat_price_in_eur": [0, 10, 0, 20, 0],
        },
    )
    assert sum_category_price(df, "price_in_eur") == sum(
        [1336.05, 680, 108.59, 3651, 95]
    )
    assert sum_category_price(df, "vat_price_in_eur") == sum([0, 10, 0, 20, 0])
    assert sum_category_price(df, "margin_price_in_eur", "departure_fees") == 32.49



import pandas as pd

from typing import Optional


def find_prices_from_entity(
    entity_df: pd.DataFrame,
    prices_df: pd.DataFrame,
    entity_key_in_prices_df: str,
    entity_key_in_entity_df="id",
) -> Optional[pd.DataFrame]:
    if len(entity_df) > 0:
        entity_id = entity_df.loc[0, entity_key_in_entity_df]
        return prices_df[prices_df[entity_key_in_prices_df] == entity_id]
    return None


def find_most_relevant_prices_factory(
    prices_propositions_by_category_df: pd.DataFrame,
    final_checks_df: pd.DataFrame,
    prices_final_check_by_category_df: pd.DataFrame,
    billings_df: pd.DataFrame,
    prices_billings_by_category_df: pd.DataFrame,
):
    def find_most_relevant_prices(
        shipment_id: str, proposition_id: str, initial_proposition=False
    ):
        if initial_proposition == True:
            final_checks_df_filtered = final_checks_df[
                final_checks_df["shipment_id"] == shipment_id
            ].reset_index(drop=True)
            prices_initial_proposition_filtered = find_prices_from_entity(
                entity_df=final_checks_df_filtered,
                prices_df=prices_propositions_by_category_df,
                entity_key_in_prices_df="proposition_id",
                entity_key_in_entity_df="initial_proposition_id",
            )
            if (
                prices_initial_proposition_filtered is not None
                and len(prices_initial_proposition_filtered) > 0
            ):
                return prices_initial_proposition_filtered

        billings_df_filtered = billings_df[billings_df["shipment_id"] == shipment_id]
        prices_billings_filtered = find_prices_from_entity(
            entity_df=billings_df_filtered,
            prices_df=prices_billings_by_category_df,
            entity_key_in_prices_df="billing_id",
        )
        if prices_billings_filtered is not None and len(prices_billings_filtered) > 0:
            return prices_billings_filtered

        final_checks_df_filtered = final_checks_df[
            final_checks_df["shipment_id"] == shipment_id
        ]
        prices_final_check_filtered = find_prices_from_entity(
            entity_df=final_checks_df_filtered,
            prices_df=prices_final_check_by_category_df,
            entity_key_in_prices_df="final_check_id",
        )
        if (
            prices_final_check_filtered is not None
            and len(prices_final_check_filtered) > 0
        ):
            return prices_final_check_filtered

        if not pd.isna(proposition_id):
            return prices_propositions_by_category_df[
                prices_propositions_by_category_df["proposition_id"] == proposition_id
            ]
        return None

    return find_most_relevant_prices



import pandas as pd
import numpy as np
from pandas.util.testing import assert_frame_equal


from ovretl.loads_utils.merge_shipments_with_loads import (
    split_propositions_shipments_loads,
    merge_with_propositions_loads_then_with_shipments_loads,
)


def test_merge_with_propositions_loads_then_with_shipments_loads():
    loads_df = pd.DataFrame(
        data={
            "total_weight": [200, 100, 200, 200],
            "total_volume": [0.9, 0.9, 1, 0.96],
            "shipment_id": [np.nan, np.nan, "2", "3"],
            "proposition_id": ["0", "1", np.nan, np.nan],
        }
    )
    shipments_df = pd.DataFrame(
        data={"shipment_id": [np.nan, "2", "3"], "proposition_id": ["0", "1", np.nan]}
    )
    shipments_df_true = pd.DataFrame(
        data={
            "shipment_id": [np.nan, "2", "3"],
            "proposition_id": ["0", "1", np.nan],
            "total_volume": [0.9, 0.9, 0.96],
            "total_weight": [200, 100, 200],
        }
    )
    (propositions_loads_df, shipments_loads_df,) = split_propositions_shipments_loads(
        loads_df
    )

    shipments_df_processed = merge_with_propositions_loads_then_with_shipments_loads(
        shipments_df=shipments_df,
        propositions_loads_df=propositions_loads_df,
        shipments_loads_df=shipments_loads_df,
    )
    assert_frame_equal(shipments_df_processed, shipments_df_true, check_dtype=False)



from ovretl.prices_utils.find_most_relevant_prices import (
    find_most_relevant_prices_factory,
)
from ovretl.prices_utils.sum_category_price import sum_category_price
from ovretl.billings_utils.compute_billing_status import compute_billing_status
from ovretl.billings_utils.extract_billing_numbers import extract_billing_numbers
from ovretl.containers_utils.merge_shipments_with_containers import *
from ovretl.containers_utils.calculate_single_container_teus import *
from ovretl.containers_utils.calculate_entity_containers import *
from ovretl.loads_utils.calculate_single_load_total_quantities import *
from ovretl.loads_utils.calculate_entity_loads import *
from ovretl.loads_utils.merge_shipments_with_loads import *
from ovretl.tracking_utils.extract_event_date import *
from ovretl.tracking_utils.join_tracking_events import *
from ovretl.employees_utils.join_employees_associations_to_employees_name import *
from ovretl.employees_utils.find_shipment_employee_name import *
from ovretl.shipowners_utils.find_shipment_category_shipowner import *
